<?php

namespace App\Database;

use App\Database\MigrationNameManipulations;

class GenerateMigration
{
	const MIGRATION_TYPE_GENERIC = 'generic';
	const MIGRATION_TYPE_COMIC = 'comic';

	protected static function generateComicStub($className)
	{
		return [
			'<?php',
			'',
			'use App\\Database\\Migration;',
			'// use App\\Database\\MigrationFields as Fld;',
			'// use App\\Database\\MigrationFieldOptions as FldOpt;',
			'// use App\\Database\\MigrationIndexes as Idx;',
			'use App\Models\Comic;',
			'',
			"class $className extends Migration",
			'{',
			'	public function migrate($dbConnection)',
			'	{',
			'		Comic::insert(',
			'			[',
			"				'title' => '',",
			"				'uri' => '',",
			"				'filename' => '',",
			"				'created_on' => '',",
			"				'description' => '',",
			"				'keywords' => '',",
			"				'featured' => 0,",
			"				'active' => 1",
			'			],',
			'			[[',
			"				'title' => '',",
			"				'url' => '',",
			"				'description' => '',",
			"				'weight' => 0,",
			"				'active' => 1",
			'			]]',
			'		);',
			'	}',
			'}'
		];
	}

	protected static function generateGenericStub($className)
	{
		return [
			'<?php',
			'',
			'use App\\Database\\Migration;',
			'use App\\Database\\MigrationFields as Fld;',
			'use App\\Database\\MigrationFieldOptions as FldOpt;',
			'use App\\Database\\MigrationIndexes as Idx;',
			'',
			"class $className extends Migration",
			'{',
			'	public function migrate($dbConnection)',
			'	{',
			'		// PLACE CODE HERE',
			'	}',
			'}'
		];
	}

	public static function generate($name, $timestamp, $migrationType = self::MIGRATION_TYPE_GENERIC)
	{
		$className = MigrationNameManipulations::migrationNameToClassName($name, $timestamp);
		switch ($migrationType)
		{
			case self::MIGRATION_TYPE_COMIC:
				$migrationStub = self::generateComicStub($className);
				break;
			case self::MIGRATION_TYPE_GENERIC:
				$migrationStub = self::generateGenericStub($className);
				break;
		}
		return implode("\n", $migrationStub);
	}

	public static function writeMigration($migrationName, $timestamp, $migrationStub)
	{
		$fileName = MigrationNameManipulations::migrationNameToFileName($migrationName, $timestamp);
		return file_put_contents(
			DATABASE_MIGRATIONS_FOLDER . $fileName,
			$migrationStub
		);
	}
}