<?php

namespace App\Database;

use PDO;

class MigrationFields
{
	const SIZE_BIG_INTEGER = 'bigint';
	const SIZE_INTEGER = 'int';
	const SIZE_TINY_INT = 'tinyint';

	protected static function internalDate($name, $type, array $options = [])
	{
		$setOptions = self::buildOptionsArray($options);
		$dateOptions = [
			$name,
			$type,
			self::option($setOptions, 'null', 'not null')
		];
		if (array_key_exists('now', $setOptions))
		{
			$dateOptions[] = $setOptions['now'];
		}
		return implode(' ', $dateOptions);
	}

	protected static function internalInteger($name, $size, array $options)
	{
		$setOptions = self::buildOptionsArray($options);
		return implode(' ', [
			$name,
			$size,
			self::option($setOptions, 'signed', 'unsigned'),
			self::option($setOptions, 'null', 'not null'),
			'default', self::option($setOptions, 'default', 0)
		]);
	}

	protected static function buildOptionsArray(array $options)
	{
		$returnOptions = [];
		foreach ($options as $option)
		{
			$optionSections = explode(':', $option, 1);
			$optionName = $optionSections[0];
			$optionValue = array_key_exists(1, $optionSections)
				? $optionSections[1]
				: '';
			$returnOptions[$optionName] = $optionValue;
		}
		return $returnOptions;
	}

	protected static function option(array $options, $name, $default)
	{
		return array_key_exists($name, $options)
			 ? $options[$name]
			 : $default;
	}

	protected static function quoteOption(array $options, $name, $default)
	{
		return '"' . self::option($options, $name, $default) . '"';
	}

	public static function autoIncrement($name, $size = self::SIZE_INTEGER)
	{
		return implode(' ', [
			$name,
			$size,
			'unsigned',
			'not null',
			'auto_increment'
		]); 
	}

	public static function bigInteger($name, array $options = [])
	{
		return self::internalInteger($name, self::SIZE_BIG_INTEGER, $options);
	}

	public static function boolean($name, $default = false)
	{
		$defaultSQL = ($default) ? 1 : 0;
		return implode(' ', [
			$name,
			self::SIZE_TINY_INT,
			'unsigned',
			'not null',
			'default', $defaultSQL
		]); 
	}

	public static function date($name, array $options = [])
	{
		return self::internalDate($name, 'date', $options);
	}

	public static function dateTime($name, array $options = [])
	{
		return self::internalDate($name, 'datetime', $options);
	}

	public static function integer($name, array $options = [])
	{
		return self::internalInteger($name, self::SIZE_INTEGER, $options);
	}

	public static function text($name, array $options = [])
	{
		$setOptions = self::buildOptionsArray($options);
		return implode(' ', [
			$name,
			'text',
			self::option($setOptions, 'null', 'not null'),
			'default', self::quoteOption($setOptions, 'default', '')
		]);
	}

	public static function tinyInteger($name, array $options = [])
	{
		return self::internalInteger($name, self::SIZE_TINY_INT, $options);
	}

	public static function varchar($name, array $options = [])
	{
		$setOptions = self::buildOptionsArray($options);
		$size = self::option($setOptions, 'size', 255);
		return implode(' ', [
			$name,
			"varchar($size)",
			self::option($setOptions, 'null', 'not null'),
			'default', self::quoteOption($setOptions, 'default', '')
		]);
	}
}