<?php

namespace App\Database;

use App\Database\CRUD;
use App\Database\MigrationNameManipulations;
use Exception;

class RunMigrations
{
	protected static function isPendingMigration($fileName, $lastMigrated)
	{
		if (preg_match('/^\d/', $fileName))
		{
			$fileNameParts = explode('-', $fileName);
			return ($fileNameParts[0] > $lastMigrated);
		}
		return false;
	}

	public static function getLastMigration()
	{
		try
		{
			$migrationTimestamp = DATABASE_MIGRATION_TIMESTAMP_FIELD;
			$migrationTable = DATABASE_MIGRATIONS_TABLE;
			$tables = dbQuery("select {$migrationTimestamp} from {$migrationTable} order by id desc limit 1");
			$records = $tables->fetchAll(); //[0][$migrationTimestamp];
			return $records[0][$migrationTimestamp];
		}
		catch (Exception $e)
		{
			return 0;
		}
	}

	public static function getPendingMigrations($lastMigratedFile = '')
	{
		$dirHandle = opendir(DATABASE_MIGRATIONS_FOLDER);
		$pendingList = [];
		while (false !== ($file = readdir($dirHandle)))
		{
			if (self::isPendingMigration($file, $lastMigratedFile))
			{
				$pendingList[] = $file;
			}
		}
		closedir($dirHandle);
		sort($pendingList);
		return $pendingList;
	}

	public static function runMigration($fileName)
	{
		require_once(DATABASE_MIGRATIONS_FOLDER . $fileName);
		$migrationClassName = MigrationNameManipulations::fileNameToClassName($fileName);
		$dbConnection = dbConnection();
		$migration = new $migrationClassName;
		$migration->beginMigration($dbConnection);
		try
		{
			$migration->migrate($dbConnection);
			$error = null;
			CRUD::insert(DATABASE_MIGRATIONS_TABLE, [
				DATABASE_MIGRATION_DEBUG_NAME => $fileName,
				DATABASE_MIGRATION_TIMESTAMP_FIELD => MigrationNameManipulations::timestampFromFileName($fileName)
			]);
		}
		catch (Exception $e)
		{
			$error = $e;
		}
		$migration->endMigration(
			$dbConnection,
			$error
		);
	}
}