<?php

namespace App\Models;

use App\Database\CRUD;
use App\Models\Link;

class Comic
{
	protected static $table = 'comics';
	protected static $landingPageColumns = 'id, title, uri, filename, description';

	public static function comic($uri)
	{
		$table = self::$table;
		$sql = "select * from $table where active = 1 and uri = ? limit 1";
		return dbQuery($sql, [$uri]);
	}

	public static function featured()
	{
		$landingPageColumns = self::$landingPageColumns;
		$table = self::$table;
		$pageSize = config(CONFIG_SECTION_NAME_COMICS)['pageSize'];
		$sql = "select $landingPageColumns from $table where active = 1 and featured = 1 order by created_on desc limit $pageSize";
		return dbQuery($sql);
	}

	public static function insert(array $comicFields, array $arrayLinkFields = [])
	{
		$comidId = CRUD::insert(self::$table, $comicFields);
		foreach ($arrayLinkFields as $linkFields)
		{
			$linkFields['comic_id'] = $comidId;
			Link::insert($linkFields);
		}
	}

	public static function list($pageIndex = 0)
	{
		$landingPageColumns = self::$landingPageColumns;
		$pageSize = config(CONFIG_SECTION_NAME_COMICS)['pageSize'];
		$numberRecords = $pageSize + 1;
		$pageOffset = intval($pageIndex) * $pageSize;
		$table = self::$table;
		$sql = "select $landingPageColumns from $table where active = 1 and featured = 0 order by created_on desc limit $pageOffset, $numberRecords";
		return dbQuery($sql);
	}

	public static function recent($pageIndex = 0)
	{
		$landingPageColumns = self::$landingPageColumns;
		$pageSize = config(CONFIG_SECTION_NAME_COMICS)['pageSize'];
		$pageOffset = intval($pageIndex) * $pageSize;
		$table = self::$table;
		$sql = "select $landingPageColumns from $table where active = 1 and featured = 0 order by created_on desc limit $pageOffset, $pageSize";
		return dbQuery($sql);
	}

	public static function related($id, $title, $description, $keywords)
	{
		$landingPageColumns = self::$landingPageColumns;
		$pageSize = config(CONFIG_SECTION_NAME_COMICS)['pageSize'];
		$table = self::$table;
		$sql = "select $landingPageColumns from $table where active = 1 and id != ? order by 1";
		$data = [$id];
		$keywords = stringsToKeywords($title, $description, $keywords);
		foreach ($keywords as $word)
		{
			$sql .= ' + (instr(title, ?) > 0)';
			$sql .= ' + (instr(description, ?) > 0)';
			$sql .= ' + (instr(keywords, ?) > 0)';
			$data[] = $word;
			$data[] = $word;
			$data[] = $word;
		}
		$sql .= " desc limit $pageSize";
		return dbQuery($sql, $data);
	}

	public static function search($searchTerm)
	{
		$landingPageColumns = self::$landingPageColumns;
		$pageSize = config(CONFIG_SECTION_NAME_COMICS)['pageSize'];
		$table = self::$table;
		$data = [];
		$keywords = stringsToKeywords($searchTerm);
		$sql = "select $landingPageColumns from $table";
		$where = [];
		foreach ($keywords as $word)
		{
			$where[] = 'instr(title, ?) + instr(description, ?) + instr(keywords, ?)';
			$data[] = $word;
			$data[] = $word;
			$data[] = $word;
		}
		$sql .= ' where active = 1 and (' . implode(' or ', $where) . ')';
		$order = [];
		foreach ($keywords as $word)
		{
			$order[] = '(instr(title, ?) > 0)';
			$order[] = '(instr(description, ?) > 0)';
			$order[] = '(instr(keywords, ?) > 0)';
			$data[] = $word;
			$data[] = $word;
			$data[] = $word;
		}
		$sql = $sql . ' order by ' . implode('+', $order) . " desc limit $pageSize";
		return dbQuery($sql, $data);
	}
}