<?php

namespace App\Routing;

use App\GlobalArrays\ServerArray;
use App\Requests\SafeArray;
use App\Routing\RouteData;

class RouteParser
{
	protected static function isRouteParameter($segment)
	{
		return '{$' === substr($segment, 0, 2);
	}

	protected static function routeParameterName($name)
	{
		return str_replace(['{$', '}'], '', $name);
	}

	protected static function uriSegments($uri)
	{
		return explode('/', trim($uri, '/'));
	}

	static function extractCurrentUri()
	{
		return ServerArray::item(ServerArray::REQUEST_URI);
	}

	static function match($routes, $uri)
	{
		$uriSegments = self::uriSegments($uri);
		foreach ($routes as $route => $controllerName)
		{
			$routePatternSegments = self::uriSegments($route);
			if (count($uriSegments) == count($routePatternSegments))
			{
				$routeParams = [];
				$match = true;
				foreach ($routePatternSegments as $segmentIndex => $segment)
				{
					$segmentValue = $uriSegments[$segmentIndex];
					if (self::isRouteParameter($segment))
					{
						$routeParams[self::routeParameterName($segment)] = $segmentValue;
					}
					else
					{
						if ($segment !== $segmentValue)
						{
							$match = false;
							break;
						}
					}
				}
				if ($match)
				{
					return new RouteData(
						$controllerName,
						new SafeArray($routeParams)
					);
				}
			}
		}
	}
}