<?php

require_once(BASE_CODE_PATH . 'constants.php');
require_once(BASE_CODE_PATH . 'settings.php');

spl_autoload_register(function ($className) {
	$classNameArray = explode('\\', $className);
	array_shift($classNameArray);
	$fullPath = APP_CODE_PATH . implode(DIRECTORY_SEPARATOR, $classNameArray) . '.php';
	require_once($fullPath);
});

use App\Database\Connection;
use App\LifeCycle\Request;
use App\Routing\RouteParser;

function currentRoute()
{
	$routeList = config(CONFIG_SECTION_NAME_ROUTES);
	$uri = RouteParser::extractCurrentUri();
}

function dbConnection()
{
	return Connection::connect();
}

function dbExecute($sql)
{
	return Connection::execute($sql);
}

function dbQuery($sql, array $data = [])
{
	return Connection::query($sql, $data);
}

function runController(Request $request, $controllerName)
{
	$controller = "App\\Controllers\\$controllerName";
	$method = $request->server->item('REQUEST_METHOD');
	$controllerMethods = get_class_methods($controller);
	$handleMethod = CONTROLLER_METHOD_BASE . ucwords(strtolower($method));
	$methodName = in_array($handleMethod, $controllerMethods)
		? $handleMethod
		: CONTROLLER_METHOD_BASE;
	(new $controller)->$methodName($request);
}

function stringsToKeywords(...$strings):array
{
	$words = implode(' ', $strings);
	$minWordLength = config(CONFIG_SECTION_NAME_SEARCH)['keywordMinLength'];
	$return = [];
	foreach (preg_split('/[\s+]+/', $words) as $word)
	{
		if ($wordTrimmed = trim($word))
		{
			if (strlen($wordTrimmed) >= $minWordLength)
			{
				$return[] = $wordTrimmed;
			}
		}
	}
	return $return;
}

// function validateRequest(Request $request):bool
// {
// 	if (in_array(VALIDATION_METHOD_NAME, get_class_methods($request)))
// 	{
// 		return call_user_func([$request, VALIDATION_METHOD_NAME], [$request]);
// 	}
// 	return true;
// }

// function validationFailedAction(BaseController $controller, Request $request)
// {
// 	if (in_array(VALIDATION_FAILED_METHOD_NAME, get_class_methods($controller)))
// 	{
// 		call_user_func([$controller, VALIDATION_FAILED_METHOD_NAME], [$request]);
// 	}
// 	header(HTTP_RESPONSE_CODE_ERROR_400);
// 	exit();
// }