<?php

define('MAIL_LIBRARY_PATH', BASE_CODE_PATH . '/third-party/PHPMailer-master/src/');
require_once(MAIL_LIBRARY_PATH . 'Exception.php');
require_once(MAIL_LIBRARY_PATH . 'PHPMailer.php');
require_once(MAIL_LIBRARY_PATH . 'SMTP.php');

use PHPMailer\PHPMailer\Exception;
use PHPMailer\PHPMailer\PHPMailer;
use PHPMailer\PHPMailer\SMTP;

function buildHtmlMessage(array $message)
{
	$content = '';
	foreach ($message as $name => $value)
	{
		$content .= '<b>' . htmlentities($name) . '</b>' . "\n";
		$content .= htmlentities($value) . "\n\n";
	}
	return implode("\n", [
		'<!doctype html>',
		'<html>',
			'<body>',
				'<table border="0" cellpadding="0" cellspacing="0">',
					'<tr>',
						'<td>',
							'<pre>' . $content . '</pre>',
						'</td>',
					'</tr>',
				'</table>',
			'</body>',
		'</html>'
	]);
}

function sendNotification($subject, $message, $fromEmail = '', $fromName = '')
{
	try
	{
		$mailConfig = config(CONFIG_SECTION_NAME_EMAIL);
		$mail = new PHPMailer(true);
		if ('true' === $mailConfig['debug'])
		{
			$mail->SMTPDebug = SMTP::DEBUG_CONNECTION;
		}
		$mail->isSMTP();
		$mail->Host = $mailConfig['smtpServer'];
		$mail->SMTPAuth = true;
		$mail->Username = $mailConfig['user'];
		$mail->Password = $mailConfig['password'];
		$mail->SMTPSecure = PHPMailer::ENCRYPTION_SMTPS;
		// $mail->SMTPSecure = PHPMailer::ENCRYPTION_STARTTLS;
		$mail->Port = $mailConfig['port'];

		$mail->setFrom(
			($fromEmail) ? $fromEmail : $mailConfig['notify'],
			($fromName) ? $fromName : $mailConfig['notify']
		);
		$mail->SMTPOptions = [
			'ssl' => [
				'verify_peer' => false,
				'allow_self_signed' => true,
				'peer_name' => $mailConfig['sslPeerName']
			],
		];
		$mail->addAddress($mailConfig['notify']);

		$mail->isHTML(true);
		$mail->Subject = $subject;
		$mail->Body = $message;
		$mail->AltBody = strip_tags($message);

		return $mail->send();
	}
	catch (Exception $e)
	{
		throw($e);
	}
}