<?php

define('CONFIG_SECTION_NAME_CACHE', 'cache');
define('CONFIG_SECTION_NAME_COMICS', 'comics');
define('CONFIG_SECTION_NAME_DATABASE', 'database');
define('CONFIG_SECTION_NAME_EMAIL', 'email');
define('CONFIG_SECTION_NAME_ERRORS', 'errors');
define('CONFIG_SECTION_NAME_FORM_SUBMISSION_KEYS', 'submissionKeys');
define('CONFIG_SECTION_NAME_PAGES', 'pages');
define('CONFIG_SECTION_NAME_ROUTES', 'routes');
define('CONFIG_SECTION_NAME_SEARCH', 'search');
define('CONFIG_SECTION_TEMPLATE_INIT', 'template_init');

define('CONTROLLER_MODELS_INCLUDE', APP_CODE_PATH . 'Models/');

define('DATABASE_MIGRATIONS_TABLE', 'migrations');
define('DATABASE_MIGRATION_DEBUG_NAME', 'migration_name');
define('DATABASE_MIGRATION_TIMESTAMP_FIELD', 'migration_timestamp');
define('DATABASE_MIGRATIONS_FOLDER', BASE_CODE_PATH . 'database/migrations/');
define('DATABASE_TIMESTAMP_FORMAT', 'Y-m-d H:i:s');

define('HASH_TYPE_DEFAULT', 'sha256');

define('TEMPLATE_PATH', BASE_CODE_PATH . 'templates/');
define('TEMPLATE_CACHE_PATH', BASE_CODE_PATH . 'cache/templates/');

function config($sectionName = '')
{
	$iniConfig = parseSettingsIniFile(BASE_CODE_PATH . 'settings.ini');
	$config = [
		CONFIG_SECTION_NAME_CACHE => $iniConfig['Cache'],
		CONFIG_SECTION_NAME_COMICS => [
			'pageSize' => 10
		],
		CONFIG_SECTION_NAME_DATABASE => $iniConfig['Database'],
		CONFIG_SECTION_NAME_EMAIL => $iniConfig['Email'],
		CONFIG_SECTION_NAME_ERRORS => [
			'level' => E_ALL
		],
		CONFIG_SECTION_NAME_FORM_SUBMISSION_KEYS => $iniConfig['Forms'],
		CONFIG_SECTION_NAME_PAGES => [
			'contact' => [
				'contactTypes' => [
					'Comment',
					'Copyright concern',
					'Question',
					'Other'
				]
			]
		],
		CONFIG_SECTION_NAME_ROUTES => [
			'/' => 'LandingController',
			'/about' => 'AboutController',
			'/comics' => 'ComicListController',
			'/comics/{$page}' => 'ComicListController',
			'/comics-links' => 'ComicsLinksController',
			'/comic/{$uri}' => 'ComicDetailController',
			'/contact' => 'ContactController',
			'/contact/thanks' => 'ThanksController',
			'/privacy' => 'PrivacyPolicyController',
			'/search' => 'SearchController',
			'/search/{$page}' => 'SearchController',
			'/terms' => 'TermsOfUseController'
		],
		CONFIG_SECTION_NAME_SEARCH => [
			'keywordMinLength' => 4
		],
		CONFIG_SECTION_TEMPLATE_INIT => [
			'comicFolders' => [
				'thumb' => '/assets/images/128/',
				'small' => '/assets/images/512/',
				'large' => '/assets/images/1024/'
			],
			'routes' => [
				'comicDetailPage' => '/comic/',
				'comicListPage' => '/comics/'
			],
			'site.name' => 'comics.scottk.net'
		]
	];

	if ($sectionName)
	{
		return array_key_exists($sectionName, $config)
			? $config[$sectionName]
			: '';
	}
	return $config;
}

function parseSettingsIniFile($filePath)
{
	$file = file_get_contents($filePath);
	$lines = explode("\n", $file);
	$return = [];
	$currentSection = '';
	foreach ($lines as $line)
	{
		if ($line)
		{
			switch (substr($line, 0, 1))
			{
				case '#':
					break;
				case '[':
					$currentSection = preg_replace('/[^a-zA-Z]/', '', $line);
					$return[$currentSection] = [];
					break;
				default:
					$lineSections = explode('=', $line, 2);
					$name = trim($lineSections[0]);
					$value = array_key_exists(1, $lineSections)
						? trim($lineSections[1])
						: '';
					$return[$currentSection][$name] = $value;
			}
		}
	}
	return $return;
}